@@ -3,7 +3,7 @@ package Module::Build::JSAN::Installable;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 use Module::Build::JSAN;
 @ISA = qw(Module::Build::JSAN);
@@ -22,6 +22,7 @@ use JSON;
 
 __PACKAGE__->add_property('task_name' => 'core');
 __PACKAGE__->add_property('static_dir' => 'static');
+__PACKAGE__->add_property('docs_markup' => 'pod');
 
 
 #================================================================================================================================================================================================================================================
@@ -201,6 +202,241 @@ sub ACTION_test {
 }
 
 
+#================================================================================================================================================================================================================================================
+sub ACTION_dist {
+    my $self = shift;
+
+    $self->depends_on('doc');
+    $self->depends_on('manifest');
+    $self->depends_on('distdir');
+
+    my $dist_dir = $self->dist_dir;
+
+    $self->_strip_pod($dist_dir);
+
+    $self->make_tarball($dist_dir);
+    $self->delete_filetree($dist_dir);
+
+#    $self->add_to_cleanup('META.json');
+#    $self->add_to_cleanup('*.gz');
+}
+
+
+
+#================================================================================================================================================================================================================================================
+sub ACTION_docs {
+    my $self = shift;
+    
+    my $markup = $self->docs_markup;
+    
+    if ($markup eq 'pod') {
+        $self->generate_docs_from_pod()
+    } elsif ($markup eq 'md') {
+        $self->generate_docs_from_md()
+    } elsif ($markup eq 'mmd') {
+        $self->generate_docs_from_mmd()
+    }
+}
+
+
+#================================================================================================================================================================================================================================================
+sub generate_docs_from_md {
+    my $self = shift;
+    
+    require Text::Markdown;
+    
+    $self->process_dist_packages({
+        html => \sub {
+            my ($comments, $content) = @_;
+            return (Text::Markdown::markdown($comments), 'html')
+        },
+        
+        md => \sub {
+            my ($comments, $content) = @_;
+            return ($comments, 'txt');
+        }
+    })
+}
+
+
+#================================================================================================================================================================================================================================================
+sub generate_docs_from_mmd {
+    my $self = shift;
+    
+    require Text::MultiMarkdown;
+    
+    $self->process_dist_packages({
+        html => sub {
+            my ($comments, $content) = @_;
+            return (Text::MultiMarkdown::markdown($comments), 'html')
+        },
+        
+        mmd => sub {
+            my ($comments, $content) = @_;
+            return ($comments, 'txt');
+        }
+    })
+}
+
+
+#================================================================================================================================================================================================================================================
+sub process_dist_packages {
+    my ($self, $convertors) = @_;
+    
+    my $lib_dir  = dir('lib');
+    
+    my $js_files = $self->find_dist_packages;
+    
+    
+    foreach my $file (map { $_->{file} } values %$js_files) {
+        my $content = file($file)->slurp;
+        
+        my $comments = $self->strip_doc_comments($content);
+
+
+        foreach my $format (keys(%$convertors)) {
+            
+            #receiving formatted docs
+            my $convertor = $convertors->{$format};
+            
+            my ($result, $result_ext) = &$convertor($comments, $content);
+            
+            
+            #preparing 'doc' directory for current format 
+            my $format_dir = catdir 'doc', $format;
+            
+            unless (-e $format_dir) {
+                File::Path::mkpath($format_dir, 0, 0755) or die "Couldn't mkdir $format_dir: $!";
+                
+                $self->add_to_cleanup($format_dir);
+            }
+            
+            
+            #saving results
+            (my $res = $file) =~ s|^$lib_dir|$format_dir|;
+            
+            $res =~ s/\.js$/.$result_ext/;
+            
+            my $res_dir = dirname $res;
+            
+            unless (-e $res_dir) {
+                File::Path::mkpath($res_dir, 0, 0755) or die "Couldn't mkdir $res_dir: $!";
+            }
+            
+            open my $fh, ">", $res or die "Cannot open $res: $!\n";
+    
+            print $fh $result;
+    
+            close $fh;
+        }
+    }
+}
+
+
+
+#================================================================================================================================================================================================================================================
+sub strip_doc_comments {
+    my ($self, $content) = @_;
+    
+    my @comments = ($content =~ m[^\s*/\*\*(.*?)\*/]msg);
+    
+    return join '', @comments; 
+}
+
+
+#================================================================================================================================================================================================================================================
+sub generate_docs_from_pod {
+    my $self = shift;
+
+    require Pod::Simple::HTML;
+    require Pod::Simple::Text;
+    require Pod::Select;
+
+    for (qw(html text pod)) {
+        my $dir = catdir 'doc', $_;
+        
+        unless (-e $dir) {
+            File::Path::mkpath($dir, 0, 0755) or die "Couldn't mkdir $dir: $!";
+            
+            $self->add_to_cleanup($dir);
+        }
+    }
+
+    my $lib_dir  = catdir 'lib';
+    my $pod_dir  = catdir 'doc', 'pod';
+    my $html_dir = catdir 'doc', 'html';
+    my $txt_dir  = catdir 'doc', 'text';
+
+    my $js_files = $self->find_dist_packages;
+    
+    foreach my $file (map { $_->{file} } values %$js_files) {
+        (my $pod = $file) =~ s|^$lib_dir|$pod_dir|;
+        
+        $pod =~ s/\.js$/.pod/;
+        
+        my $dir = dirname $pod;
+        
+        unless (-e $dir) {
+            File::Path::mkpath($dir, 0, 0755) or die "Couldn't mkdir $dir: $!";
+        }
+        
+        # Ignore existing documentation files.
+        next if -e $pod;
+        
+        
+        open my $fh, ">", $pod or die "Cannot open $pod: $!\n";
+
+        Pod::Select::podselect( { -output => $fh }, $file );
+
+        print $fh "\n=cut\n";
+
+        close $fh;
+    }
+    
+
+    for my $pod (@{Module::Build->rscan_dir($pod_dir, qr/\.pod$/)}) {
+        # Generate HTML docs.
+        (my $html = $pod) =~ s|^\Q$pod_dir|$html_dir|;
+        
+        $html =~ s/\.pod$/.html/;
+        
+        my $dir = dirname $html;
+        
+        unless (-e $dir) {
+            File::Path::mkpath($dir, 0, 0755) or die "Couldn't mkdir $dir: $!";
+        }
+        
+        open my $fh, ">", $html or die "Cannot open $html: $!\n";
+        
+        my $parser = Pod::Simple::HTML->new;
+        $parser->output_fh($fh);
+        $parser->parse_file($pod);
+        
+        close $fh;
+
+        # Generate text docs.
+        (my $txt = $pod) =~ s|^\Q$pod_dir|$txt_dir|;
+        
+        $txt =~ s/\.pod$/.txt/;
+        
+        $dir = dirname $txt;
+        
+        unless (-e $dir) {
+            File::Path::mkpath($dir, 0, 0755) or die "Couldn't mkdir $dir: $!";
+        }
+        
+        open $fh, ">", $txt or die "Cannot open $txt: $!\n";
+        
+        $parser = Pod::Simple::Text->new;
+        $parser->output_fh($fh);
+        $parser->parse_file($pod);
+        
+        close $fh;
+    }
+}
+
+
+
 __PACKAGE__ # nothingmuch (c) 
 
 __END__
@@ -227,7 +463,8 @@ In F<Build.PL>:
           'Test.Simple' => 0.20,
       },
       
-      static_dir => 'assets'
+      static_dir => 'assets',
+      docs_markup => 'mmd'
   );
 
   $build->create_build_script;
@@ -330,7 +567,16 @@ JSAN library. This way you can access any module from it, with URLs like:
 B<'/jsan/Test/Run.js'>  
 
 
-=item 2 ./Build task [--task_name=foo]
+=item 2 ./Build docs
+
+This action will build a documentation files for this distribution. Default markup for documentation is POD. Alternative markup 
+can be specified with B<docs_markup> configuration parameter (see Synopsis). Currently supported markups: 'pod', 
+'md' (Markdown via Text::Markdown), 'mmd' (MultiMarkdown via Text::MultiMarkdown). 
+
+Resulting documentation files will be placed under B</docs> directory, categorized by the formats. For 'pod' markup there will be
+/doc/html, /doc/pod and /doc/text directories. For 'md' and 'mmd' markups there will be /doc/html and /doc/[m]md directories.
+
+=item 3 ./Build task [--task_name=foo]
 
 This action will build a specific concatenated version (task) of current distribution.
 Default task name is B<'core'>, task name can be specified with B<--task_name> command line option.
@@ -342,7 +588,7 @@ After concatenation, resulting file is placed on the following path: B</lib/Task
 considering the name of your distribution was B<Distribution::Name> and the task name was B<sample_task>
 
 
-=item 3 ./Build test
+=item 4 ./Build test
 
 This action relies on not yet released JSAN::Prove module, stay tuned for further updates.
 
